#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  simul.py
#
#  Copyright 2024 john <jcoppens@vostro.ampr.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

# Cambios:
#
# 2024/03/13:
#   Por motivos de claridad:
#       - La carga del capacitor se guarda en la propiedad del capacitor en
#         Coulomb (no en voltaje como hicimos en clase)
#       - El valor de la resistencia y del capacitor se guarda en la
#         propiedad '.value'

import pylab as plt
from pdb import set_trace as st
from discrete import Capacitor, Resistor
from logic import Buffer, Inverter

dT = 0.005          # en segundos
Nsamples = 200      # Número de muestras


def plot_schmitt_trigger():
    """ Grafica el comportamento de un oscilador sencillo con una compuerta
        Inverter con Scmitt Trigger (ej. CD40106), realimentado con 100kOhm y
        con un capacitor de 1µF
    """
    gate = Inverter(v_thr_hi = 4.2)
    c = Capacitor(1e-6)
    r = Resistor(1e5)
    x = []; y1 = []; y2 = []

    for i in range(Nsamples+1):
        v_out = gate.output(c.voltage())
        x.append(i*dT)
        y1.append(c.voltage())
        y2.append(v_out)
        i_r = r.current(v_out - c.voltage())
        c.discharge(-i_r, dT)

    plt.plot(x, y1, label = 'Capacitor')
    plt.plot(x, y2, label = 'Salida digital')
    plt.title('Oscilador con Schmitt Trigger')
    plt.legend(loc = 'lower right')
    plt.xlabel('Entrada')
    plt.ylabel('Salida')
    plt.grid(True)
    plt.show()



def main(args):
    plot_schmitt_trigger()
    return 0



if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv))
